part of 'cell_service.dart';

typedef GridCellMap = LinkedHashMap<String, GridCell>;

class _GridCellCacheObject {
  GridCellCacheKey key;
  dynamic object;
  _GridCellCacheObject({
    required this.key,
    required this.object,
  });
}

class GridCellCacheKey {
  final String fieldId;
  final String rowId;
  GridCellCacheKey({
    required this.fieldId,
    required this.rowId,
  });
}

class GridCellsCache {
  final String gridId;

  /// fieldId: {cacheKey: cacheData}
  final Map<String, Map<String, dynamic>> _cellDataByFieldId = {};
  GridCellsCache({
    required this.gridId,
  });

  void remove(String fieldId) {
    _cellDataByFieldId.remove(fieldId);
  }

  void insert<T extends _GridCellCacheObject>(T item) {
    var map = _cellDataByFieldId[item.key.fieldId];
    if (map == null) {
      _cellDataByFieldId[item.key.fieldId] = {};
      map = _cellDataByFieldId[item.key.fieldId];
    }

    map![item.key.rowId] = item.object;
  }

  T? get<T>(GridCellCacheKey key) {
    final map = _cellDataByFieldId[key.fieldId];
    if (map == null) {
      return null;
    } else {
      final object = map[key.rowId];
      if (object is T) {
        return object;
      } else {
        if (object != null) {
          Log.error("Cache data type does not match the cache data type");
        }

        return null;
      }
    }
  }

  Future<void> dispose() async {
    _cellDataByFieldId.clear();
  }
}
